@@ -1,6 +1,6 @@
 =head1 NAME
 
-Log::Handler - Log messages to one or more outputs.
+Log::Handler - Log messages to several outputs.
 
 =head1 SYNOPSIS
 
@@ -16,13 +16,37 @@ Log::Handler - Log messages to one or more outputs.
         newline  => 1,
     });
 
-    $log->warning("a warinng here");
+    $log->warning("warn message");
+
+Or
+
+    # init myapp logger with accessor LOG
+    package MyApp;
+    use Log::Handler myapp => 'LOG';
+
+    LOG->add(screen => { maxlevel => 'info' });
+    LOG->info("info message");
+
+    # import myapp logger with accessor LOG
+    package MyApp::Foo;
+    use Log::Handler myapp => 'LOG';
+    LOG->info("info message from MyApp::Foo");
+
+    # get myapp logger with get_logger()
+    package MyApp::Bar;
+    use Log::Handler;
+    my $log = Log::Handler->get_logger('myapp');
+    $log->info("info message from MyApp::Bar");
 
 =head1 DESCRIPTION
 
-This module is just a simple object oriented log handler and very easy to use.
-It's possible to define a log level for your programs and control the amount
-of informations that are logged to one or more outputs.
+The C<Log::Handler> is a object oriented handler for logging, tracing and
+debugging. It is very easy to use and provides a simple interface for
+multiple output objects with lots of configuration parameters. You can
+easily filter the amount of logged information on a per-output base,
+define priorities and create patterns to format the messages.
+
+See the documentation for details.
 
 =head1 LOG LEVELS
 
@@ -31,7 +55,7 @@ There are eigth levels available:
     7   debug
     6   info
     5   notice
-    4   warning
+    4   warning, warn
     3   error, err
     2   critical, crit
     1   alert
@@ -39,6 +63,9 @@ There are eigth levels available:
 
 C<debug> is the highest and C<emergency> is the lowest level.
 
+Level C<debug> is the highest level because it basically says to log
+every peep.
+
 =head1 METHODS
 
 =head2 new()
@@ -86,20 +113,20 @@ You can set both to 8 or C<nothing> if you want to disable the logging machine.
 
 =item B<timeformat>
 
-The option C<timeformat> is used for the placeholder C<%T>. You can set
-C<timeformat> with a date and time format that is converted with
-C<POSIX::strftime>. The default format is S<"%b %d %H:%M:%S"> and looks like
+The option C<timeformat> is used to set the format for the placeholder C<%T>.
+The string is converted with C<POSIX::strftime>. The default format is set to
+S<"%b %d %H:%M:%S"> and looks like
 
     Feb 01 12:56:31
 
-As example the format S<"%Y/%m/%d %H:%M:%S"> would looks like
+If you would set the format to S<"%Y/%m/%d %H:%M:%S"> it would looks like
 
     2007/02/01 12:56:31
 
 =item B<dateformat>
 
-This options works like C<timeformat>. It's useful if you want to split the
-date and time:
+This options works like C<timeformat>. You can set a format that is used for
+the placeholder C<%D>. It's just useful if you want to split the date and time:
 
     $log->add(file => {
         filename       => 'file.log',
@@ -110,7 +137,7 @@ date and time:
 
     $log->error("an error here");
 
-Would log
+This looks like
 
     2007-02-01 12:56:31 ERROR an error here
 
@@ -118,15 +145,29 @@ This option is not used by default.
 
 =item B<newline>
 
-This helpful option appends a newline to the output message if a newline not
-exist.
+C<newline> is a very helpful option. It let the logger appends a newline to
+the message if a newline doesn't exist.
 
-    0 - to disable it (default)
-    1 - to enable it
+    0 - do nothing (default)
+    1 - append a newline if not exist
+
+Example:
+
+    $log->add(
+        screen => {
+            newline  => 1,
+            maxlevel => 'info',
+        }
+    );
+
+    $log->info("message\n");
+    $log->info("message");
+
+In both cases the message would be logged with a newline at the end.
 
 =item B<message_layout>
 
-With this option you can define your own message layout with different
+With this option it's possible to create your own message layout with different
 placeholders in C<printf()> style. The available placeholders are:
 
     %L   Log level
@@ -134,15 +175,17 @@ placeholders in C<printf()> style. The available placeholders are:
     %D   Date (option dateformat)
     %P   PID
     %H   Hostname
+    %U   User name
+    %G   Group name
     %N   Newline
     %S   Program name
-    %R   Runtime in seconds since program start
     %C   Caller - filename and line number
     %p   Caller - package name
     %f   Caller - file name
     %l   Caller - line number
     %s   Caller - subroutine name
-    %t   Time measurement - replaced with the time since the last call of the handler
+    %r   Runtime in seconds since program start
+    %t   Time measurement - replaced with the time since the last call of $log->$level
     %m   Message
     %%   Procent
 
@@ -176,8 +219,8 @@ You can create your own placeholders with the method C<set_pattern()>.
 
 This option is just useful if you want to forward messages to output
 modules that needs the parts of a message as a hash reference - as
-example L<Log::Handler::Output::Forward>, L<Log::Handler::Output::DBI>
-or L<Log::Handler::Output::Screen>.
+example L<Log::Handler::Output::Forward>, L<Log::Handler::Output::DBI>,
+L<Log::Handler::Output::Email> or L<Log::Handler::Output::Screen>.
 
 The option expects a list of placeholders:
 
@@ -186,7 +229,7 @@ The option expects a list of placeholders:
 
     # or as a string
     message_pattern => '%T %L %H %m'
- 
+
 The patterns will be replaced with real names as hash keys.
 
     %L   level
@@ -194,8 +237,10 @@ The patterns will be replaced with real names as hash keys.
     %D   date
     %P   pid
     %H   hostname
+    %U   user
+    %G   group
     %N   newline
-    %R   runtime
+    %r   runtime
     %C   caller
     %p   package
     %f   filename
@@ -230,6 +275,38 @@ Here a full code example:
         print "Message:   $msg->{message}\n";
     }
 
+=item B<prepare_message>
+
+C<prepare_message> is useful if you want to do something with the message before
+it will be logged... maybe you want to create your own layout because message_layout
+doesn't meet your claim.
+
+    $log->add(
+        screen => {
+            newline => 1,
+            message_layout  => '%m (%t)',
+            message_pattern => [ qw/%T %L %H %m/ ],
+            prepare_message => \&format,
+        }
+    );
+
+    $log->error("foo");
+    $log->error("bar");
+    $log->error("baz");
+
+    sub format {
+        my $m = shift;
+
+        $m->{message} = sprintf('%-20s %-20s %-20s %s',
+            $m->{time}, $m->{level}, $m->{hostname}, $m->{message});
+    }
+
+The output looks like
+
+    Mar 08 15:14:20      ERROR                h1434036             foo (0.039694)
+    Mar 08 15:14:20      ERROR                h1434036             bar (0.000510)
+    Mar 08 15:14:20      ERROR                h1434036             baz (0.000274)
+
 =item B<priority>
 
 With this option you can set the priority of your output objects. This means
@@ -239,16 +316,16 @@ increased +1 with each output. Example:
 
 We add a output with no priority
 
-    $log->add(file => { filename => 'file.log' });
+    $log->add(file => { filename => 'file1.log' });
 
 This output gets the priority of 10. Now we add another output
 
-    $log->add(file => { filename => 'file.log' });
+    $log->add(file => { filename => 'file2.log' });
 
 This output gets the priority of 11... and so on.
 
-Messages would be logged at first to the output with the priority of 10 and to
-the output with the priority of 11. Now you can add another output and set the
+Messages would be logged at first to the output with the priority of 10 and then
+to the output with the priority of 11. Now you can add another output and set the
 priority to 1.
 
     $log->add(screen => { dump => 1, priority => 1 });
@@ -271,18 +348,21 @@ If you set C<die_on_errors> to 0 then you have to controll it yourself.
     # or Log::Handler::errstr()
     # or $Log::Handler::ERRSTR
 
-=item B<filter>
+=item B<filter_message>
 
-With this option it's possible to set a filter for the outputs. If the filter
-is set then only messages will be logged that match the filter. You can pass a
-regexp, a code reference or a simple string. Example:
+With this option it's possible to set a filter. If the filter is set then
+only messages will be logged that match the filter. You can pass a regexp,
+a code reference or a simple string. Example:
 
     $log->add(file => {
         filename => 'file.log',
         mode     => 'append',
         newline  => 1,
         maxlevel => 6,
-        filter   => qr/log this/, # log only messages that contain 'log this'
+        filter_message => qr/log this/,
+        # or
+        # filter_message => 'log this',
+        # filter_message => '^log only this$',
     });
 
     $log->info('log this');
@@ -295,7 +375,7 @@ If you pass your own code then you have to check the message yourself.
         mode     => 'append',
         newline  => 1,
         maxlevel => 6,
-        filter   => \&my_filter
+        filter_message => \&my_filter
     });
 
     # return TRUE if you want to log the message, FALSE if not
@@ -312,7 +392,7 @@ hash reference with the options C<matchN> and C<condition>. Example:
         mode     => 'append',
         newline  => 1,
         maxlevel => 6,
-        filter   => {
+        filter_message => {
             match1    => 'log this',
             match2    => qr/with that/,
             match3    => '(?:or this|or that)',
@@ -326,6 +406,41 @@ NOTE that re-eval in regexes is not valid! Something like
 
 would cause an error!
 
+=item B<filter_caller>
+
+You can use this option to set a package name. Only messages from this
+packages will be logged.
+
+Example:
+
+    my $log = Log::Handler->new();
+
+    $log->add(screen => {
+        maxlevel => 'info',
+        newline  => 1,
+        filter_caller  => qr/^Foo::Bar\z/,
+        # or
+        # filter_caller => '^Foo::Bar\z',
+    });
+
+    package Foo::Bar;
+    $log->info('log this');
+
+    package Foo::Baz;
+    $log->info('but not that');
+
+    1;
+
+This would only log the message from the package C<Foo::Bar>.
+
+=item B<except_caller>
+
+This option is just the opposite of C<filter_caller>.
+
+If you want to log messages from all callers but C<Foo::Bar>:
+
+    except_caller => qr/^Foo::Bar\z/
+
 =item B<alias>
 
 You can set an alias if you want to get the output object later. Example:
@@ -339,7 +454,7 @@ You can set an alias if you want to get the output object later. Example:
 
     my $screen = $log->output('screen-out');
 
-    $screen->log('foo');
+    $screen->log(message => 'foo');
 
     # or in one step
 
@@ -347,9 +462,9 @@ You can set an alias if you want to get the output object later. Example:
 
 =item B<debug_trace>
 
-You can activate a debugger that writes C<caller()> informations for each log
-level that would logged. The debugger is logging all defined values except
-C<hints> and C<bitmask>. Set C<debug_trace> to 1 to activate the debugger.
+You can activate a debugger that writes C<caller()> informations for each
+active log level. The debugger is logging all defined values except C<hints>
+and C<bitmask>. Set C<debug_trace> to 1 to activate the debugger.
 The debugger is set to 0 by default.
 
 =item B<debug_mode>
@@ -378,7 +493,7 @@ The line mode looks like this:
 
 Output:
 
-    Apr 26 12:54:11 [WARN] 
+    Apr 26 12:54:11 [WARNING]
        CALL(4): package(main) filename(./trace.pl) line(15) subroutine(main::test2) hasargs(0)
        CALL(3): package(main) filename(./trace.pl) line(13) subroutine(main::test1) hasargs(0)
        CALL(2): package(main) filename(./trace.pl) line(12) subroutine(Log::Handler::__ANON__) hasargs(1)
@@ -391,7 +506,7 @@ The same code example but the debugger in block mode would looks like this:
 
 Output:
 
-   Apr 26 12:52:17 [DEBUG] 
+   Apr 26 12:52:17 [DEBUG]
       CALL(4):
          package     main
          filename    ./trace.pl
@@ -424,24 +539,10 @@ Output:
          hasargs     1
          wantarray   0
 
-Another way to enable the debugger very shortly is ...
-
-    {
-        local $Log::Handler::TRACE = 1;
-        $log->info('backtrace');
-    }
-
 =item B<debug_skip>
 
 This option let skip the C<caller()> informations the count of C<debug_skip>.
 
-    debug_skip => 2
-
-    Apr 26 12:55:07 [DEBUG] 
-       CALL(2): package(main) filename(./trace.pl) line(16) subroutine(main::test2) hasargs(0)
-       CALL(1): package(main) filename(./trace.pl) line(14) subroutine(main::test1) hasargs(0)
-       CALL(0): package(main) filename(./trace.pl) line(13) subroutine(Log::Handler::__ANON__) hasargs(1)
-
 =back
 
 =head2 HowTo use add()
@@ -449,11 +550,11 @@ This option let skip the C<caller()> informations the count of C<debug_skip>.
 The method C<add()> excepts 2 parts of options; the options for the handler and
 the options for the output module you want to use. The output modules got it's own
 documentation for all options. As example if you want to add a file-output then
-take a look into the documentation of L<Log::Handler::Output::File>.
+take a look into the documentation of L<Log::Handler::Output::File> to see which
+options are available.
 
-There are different ways to add a new output to the handler. The one way is
-to create the output object yourself and pass it with the handler options
-to C<add()>.
+There are different ways to add a new output to the handler. One way is to create
+the output object yourself and pass it with the handler options to C<add()>.
 
 Example:
 
@@ -502,7 +603,10 @@ in one step, you just need to tell the handler what do you want to add.
     my $log = Log::Handler->new();
 
     $log->add(
-        file => { # what do you want to add
+
+        # what do you want to add - a file output
+        file => {
+
             # handler options
             timeformat      => '%Y/%m/%d %H:%M:%S',
             newline         => 1,
@@ -513,6 +617,7 @@ in one step, you just need to tell the handler what do you want to add.
             debug_trace     => 0,
             debug_mode      => 2,
             debug_skip      => 0,
+
             # file options
             filename        => 'file.log',
             filelock        => 1,
@@ -522,10 +627,11 @@ in one step, you just need to tell the handler what do you want to add.
             autoflush       => 1,
             permissions     => '0660',
             utf8            => 1,
+
         }
     );
 
-The options will be splitted intern and you don't need to split it yourself,
+The options will be splitted internal and you don't need to split it yourself,
 only if you want to do it yourself.
 
 Take a look to L<Log::Handler::Examples> for more informations.
@@ -560,7 +666,7 @@ Or maybe:
 
     $log->info("Hello World!", "How are you?");
 
-Both calls would log - if the level INFO is active:
+Both calls would log - if level INFO is active:
 
     Feb 01 12:56:31 [INFO] Hello World! How are you?
 
@@ -574,7 +680,7 @@ Both calls would log - if the level INFO is active:
 
 =item B<is_notice()>
 
-=item B<is_warning()>
+=item B<is_warning()>, B<is_warn()>
 
 =item B<is_error()>, B<is_err()>
 
@@ -586,37 +692,42 @@ Both calls would log - if the level INFO is active:
 
 =back
 
-These thirteen methods could be very useful if you want to kwow if the current
+These twelve methods could be very useful if you want to kwow if the current
 level would log the message. All methods returns TRUE if the current set
 of C<minlevel> and C<maxlevel> would log the message and FALSE if not.
 
-The following example would dump the hash in any case and pass it to the
-log handler:
-
-    $log->debug(Dumper(\%hash));
-
-But that is not that what we really want! We want to dump the hash only if
-the current log level would log it.
-
-    if ( $log->is_debug ) {
-        $log->debug(Dumper(\%hash));
-    }
-
-The methods C<is_err()>, C<is_crit()> and C<is_emerg()> are just shortcuts.
-
 =head2 Other level methods
 
-There exists a lot of other level methods.
+There exists some other level methods.
 
 For a full list take a look into the documentation of L<Log::Handler::Levels>.
 
 =head2 output()
 
-Call C<output($alias)> to get the output object that you added with 
+Call C<output($alias)> to get the output object that you added with
 the option C<alias>.
 
+It's possible to access a output directly:
+
+    $log->output($alias)->log(message => 'booo');
+
 For more informations take a look to the option C<alias>.
 
+=head2 flush()
+
+Call C<flush()> if you want to send flush to all outputs that can flush.
+
+Flush means to flush buffers and/or close and re-open outputs.
+
+If you want to send it only to some outputs you can pass the aliases.
+
+    $log->flush(); # flush all
+    $log->flush('foo', 'bar'); # flush only foo and bar
+
+If option S<"die_on_errors"> is set to 0 then you can intercept errors with:
+
+    $log->flush or die $log->errstr;
+
 =head2 errstr()
 
 Call C<errstr()> if you want to get the last error message. This is useful
@@ -650,7 +761,7 @@ C<add()> fails because on missing or wrong settings!
 
 With this method it's possible to load your output configuration from a file.
 
-    $log->config(filename => 'file.conf');
+    $log->config(config => 'file.conf');
 
 Or
 
@@ -676,7 +787,7 @@ Or
         }
     });
 
-The key S<"default"> can be used to define default parameters for all file
+The key S<"default"> is used here to define default parameters for all file
 outputs. All other keys (C<error_log>, C<common_log>) are used as aliases.
 
 Take a look into the documentation of L<Log::Handler::Config> for more
@@ -686,23 +797,143 @@ informations.
 
 With this option you can set your own placeholders. Example:
 
-    $log->set_pattern('%X', 'name', sub { 'value' });
+    $log->set_pattern('%X', 'key_name', sub { 'value' });
 
     # or
 
-    $log->set_pattern('%X', 'name', 'value');
+    $log->set_pattern('%X', 'key_name', 'value');
 
 Then you can use this pattern in your message layout:
 
-    $log->add(forward => {
+    $log->add(file => {
         filename        => 'file.log',
-        message_layout  => '%X %m',
+        mode            => 'append',
+        message_layout  => '%X %m%N',
     });
 
+Or use it with C<message_pattern>:
+
+    sub func {
+        my $m = shift;
+        print "$m->{key_name} $m->{message}\n";
+    }
+
+    $log->add(forward => {
+        forward_to      => \&func,
+        message_pattern => '%X %m',
+    });
+
+Note: valid character for the key name are: C<[%\w\-\.]+>
+
+=head2 set_level()
+
+With this method it's possible to change the log level at runtime.
+
+To change the log level it's neccessary to use a alias - see option C<alias>.
+
+    $log->set_level(
+        $alias => { # option alias
+            minlevel => $new_minlevel,
+            maxlevel => $new_maxlevel,
+        }
+    );
+
+=head2 create_logger()
+
+C<create_logger()> is the same like C<new()> but it creates a global
+logger.
+
+    my $log = Log::Handler->create_logger('myapp');
+
+If you want to create more than one object you can call
+
+    my @logger = Log::Handler->create_logger('myapp1', 'myapp2', ...);
+
+=head2 get_logger()
+
+With C<get_logger()> it's possible to get a logger that was created
+with C<create_logger()> or with
+
+    use Log::Handler 'myapp';
+
+Just call
+
+    my $log = Log::Handler->get_logger('myapp');
+
+Or
+
+    my @logger = Log::Handler->get_logger('myapp1', 'myapp2', ...);
+
+=head1 GLOBAL LOG HANDLER
+
+Since version C<0.50> it's possible to define a application logger.
+This means to create a C<Log::Handler> object and import it into
+all modules of your program.
+
+    use Log::Handler alias => accessor;
+
+The accessor is created into the namespace of the caller and
+referenced to the alias.
+
+If the alias doesn't exists then a new C<Log::Handler> object
+will be created.
+
+If no accesseor is set then no accessor is exported into the
+namespace of the caller.
+
+Example:
+
+    package MyAPP;
+    use strict;
+    use warnings;
+    use Log::Handler myapp => 'LOG';
+
+    LOG->config(config => 'myapp.conf');
+    LOG->info('hello world');
+
+Now you can access the logger after you import it.
+
+    package MyAPP::Foo; 
+    use Log::Handler myapp => 'LOG';
+
+    LOG->info('message');
+
+If you want to import another accessor name ...
+
+    package MyAPP::Bar;
+    use Log::Handler myapp => 'MYLOG';
+
+    MYLOG->info('message');
+
+Another way to get the logger is to call C<get_logger()> if you
+don't want to create an accessor.
+
+    package MyAPP::Foo::Bar;
+    use Log::Handler;
+
+    my $log = Log::Handler->get_logger('myapp');
+    $log->info('message');
+
+For a little example you can take a look into the examples directory
+of the distribution (Log-Handler-$VERSION/examples/logger.pl).
+
 =head1 EXAMPLES
 
 L<Log::Handler::Examples>
 
+=head1 BENCHMARK
+
+Run examples/benchmark/benchmark.pl on a Dual-Core AMD Opteron(tm) Processor 1212 HE (1000 MHz)
+
+    simple pattern output took     :  2 wallclock secs ( 2.29 usr +  0.02 sys =  2.31 CPU) @ 43290.04/s (n=100000)
+    default pattern output took    :  4 wallclock secs ( 3.36 usr +  0.76 sys =  4.12 CPU) @ 24271.84/s (n=100000)
+    complex pattern output took    :  6 wallclock secs ( 4.85 usr +  0.91 sys =  5.76 CPU) @ 17361.11/s (n=100000)
+    message pattern output took    :  5 wallclock secs ( 4.31 usr +  0.82 sys =  5.13 CPU) @ 19493.18/s (n=100000)
+    suppressed output took         :  0 wallclock secs ( 0.17 usr +  0.00 sys =  0.17 CPU) @ 588235.29/s (n=100000)
+    filtered caller output took    :  5 wallclock secs ( 3.97 usr +  0.81 sys =  4.78 CPU) @ 20920.50/s (n=100000)
+    suppressed caller output took  :  1 wallclock secs ( 1.29 usr +  0.00 sys =  1.29 CPU) @ 77519.38/s (n=100000)
+    filtered messages output took  :  5 wallclock secs ( 3.91 usr +  0.84 sys =  4.75 CPU) @ 21052.63/s (n=100000)
+
 =head1 EXTENSIONS
 
 Send me a mail if you have questions.
@@ -757,15 +988,9 @@ IRC: irc.perl.org#perl
 
 If you send me a mail then add Log::Handler into the subject.
 
-=head1 TODO
-
-Maybe;
-
-    * Log::Handler::Output::HTML
-
 =head1 COPYRIGHT
 
-Copyright (C) 2007 by Jonny Schulz. All rights reserved.
+Copyright (C) 2007-2008 by Jonny Schulz. All rights reserved.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -807,11 +1032,23 @@ use Log::Handler::Config;
 use Log::Handler::Pattern;
 use base qw(Log::Handler::Levels);
 
-our $VERSION  = '0.39_17';
-our $ERRSTR   = '';
-our $DEBUG    = 0;
-our $TRACE    = 0;
+our $VERSION = '0.51_01';
+our $ERRSTR  = '';
 
+# $TRACE and $CALLER_LEVEL are both used as global
+# variables in other packages as well. You shouldn't
+# manipulate them if you don't know what you do.
+#
+# $TRACE is used to turn on/off tracing.
+#
+# $CALLER_LEVEL is used to determine the current caller level
+our $CALLER_LEVEL = 0;
+our $TRACE        = 0;
+
+# safe logger by app
+my %LOGGER;
+
+# Some constants...
 use constant PRIORITY => 10;
 use constant BOOL_RX  => qr/^[01]\z/;
 use constant NUMB_RX  => qr/^\d+\z/;
@@ -820,7 +1057,7 @@ use constant LEVEL_RX => qr/^(?:
     7 | debug     |
     6 | info      |
     5 | notice    |
-    4 | warning   |
+    4 | warning   | warn  |
     3 | error     | err   |
     2 | critical  | crit  |
     1 | alert     |
@@ -828,26 +1065,26 @@ use constant LEVEL_RX => qr/^(?:
         fatal
 )\z/x;
 
-# to convert minlevel and maxlevel
-my %LEVEL_BY_STRING = ( 
-    DEBUG     =>  7,  
-    INFO      =>  6,  
-    NOTICE    =>  5,  
-    NOTE      =>  5,  
-    WARNING   =>  4,  
-    WARN      =>  4,  
-    ERROR     =>  3,  
-    ERR       =>  3,  
-    CRITICAL  =>  2,  
-    CRIT      =>  2,  
-    ALERT     =>  1,  
-    EMERGENCY =>  0,  
-    EMERG     =>  0,  
+# to convert minlevel and maxlevel to a number
+our %LEVEL_BY_STRING = (
+    DEBUG     =>  7,
+    INFO      =>  6,
+    NOTICE    =>  5,
+    WARNING   =>  4,
+    WARN      =>  4,
+    ERROR     =>  3,
+    ERR       =>  3,
+    CRITICAL  =>  2,
+    CRIT      =>  2,
+    ALERT     =>  1,
+    EMERGENCY =>  0,
+    EMERG     =>  0,
     FATAL     =>  0,
 );
 
-# to iterate from minlevel to maxlevel
-my @LEVEL_BY_NUM = qw(
+# to iterate from minlevel to maxlevel and
+# create an HoA with all active levels
+our @LEVEL_BY_NUM = qw(
     EMERGENCY
     ALERT
     CRITICAL
@@ -859,6 +1096,7 @@ my @LEVEL_BY_NUM = qw(
     NOTHING
 );
 
+# shortcuts for each output
 our %AVAILABLE_OUTPUTS = (
     file    => 'Log::Handler::Output::File',
     email   => 'Log::Handler::Output::Email',
@@ -868,20 +1106,67 @@ our %AVAILABLE_OUTPUTS = (
     socket  => 'Log::Handler::Output::Socket',
 );
 
+# use Log::Handler foo => 'LOGFOO', bar => 'LOGBAR';
+# use Log::Handler qw/foo LOGFOO bar LOGBAR/;
+sub import {
+    return unless @_ > 1;
+    my $class  = shift;
+    my %create = @_ > 1 ? @_ : (@_, undef);
+    my $caller = (caller)[0];
+
+    while (my ($appl, $export) = each %create) {
+        my $logger = ();
+
+        if (!exists $LOGGER{$appl}) {
+            $LOGGER{$appl} = __PACKAGE__->new();
+        }
+
+        if ($export) {
+            no strict 'refs';
+            my $method = $caller.'::'.$export;
+            *{$method} = sub { $LOGGER{$appl} };
+        }
+    }
+}
+
+sub get_logger {
+    @_ > 1 || croak 'Usage: Log::Handler->get_logger($app)';
+    my $class = shift;
+
+    foreach my $logger (@_) {
+        if (!exists $LOGGER{$logger}) {
+            croak "logger '$logger' does not exists";
+        }
+    }
+
+    return @LOGGER{@_};
+}
+
+sub create_logger {
+    @_ > 1 || croak 'Usage: Log::Handler->create_logger($app)';
+    my $class = shift;
+
+    foreach my $logger (@_) {
+        $LOGGER{$logger} = __PACKAGE__->new();
+    }
+
+    return @LOGGER{@_};
+}
+
 sub new {
     my $class = shift;
 
-    # for full backward compatibilities
+    # for full backward compatibilities to v0.38
     if (@_) {
         require Log::Handler::Simple;
         return  Log::Handler::Simple->new(@_);
     }
 
-
     my $self = bless {
         priority => PRIORITY,   # start priority
         levels   => { },        # outputs stored by active levels
         alias    => { },        # outputs stored by an alias
+        output   => [ ],        # all outputs needed for flush()
         pattern  =>             # default pattern
             &Log::Handler::Pattern::get_pattern,
     }, $class;
@@ -890,7 +1175,7 @@ sub new {
 }
 
 sub add {
-    @_ == 3 or Carp::croak 'Usage: add( type => \%options )';
+    @_ == 3 or Carp::croak 'Usage: $log->add( $output => \%options )';
     my $self   = shift;
     my $output = $self->_new_output(@_);
     my $levels = $self->{levels};
@@ -902,7 +1187,8 @@ sub add {
     # hash-tree $self->{levels}->{INFO}. On this way it's possible
     # to check very fast if the level is active
     #
-    #   if (exists $self->{levels}->{INFO}) { ... }
+    #   my $levels = $self->{levels};
+    #   if (exists $levels->{INFO}) { ... }
     #
     # and loop over all output objects and pass the message to it.
 
@@ -931,15 +1217,20 @@ sub add {
         $self->{alias}->{$alias} = $output;
     }
 
+    # save all outputs here
+    push @{$self->{outputs}}, $output;
+
     return 1;
 }
 
 sub config {
+    @_ > 1 or Carp::croak 'Usage: $log->config( %param )';
     my $self = shift;
     my $configs = Log::Handler::Config->config(@_);
 
     # Structure:
     #   $configs->{file} = [ output configs ];
+    #   $configs->{dbi}  = [ output configs ];
 
     while ( my ($type, $config) = each %$configs ) {
         for my $c (@$config) {
@@ -951,33 +1242,129 @@ sub config {
 }
 
 sub set_pattern {
-    @_ == 4 or Carp::croak 'Usage: $log->set_pattern( $pattern, $name, $code )';
-    my ($self, $pattern, $name, $proto) = @_;
+    (@_ == 3 || @_ == 4)
+        or Carp::croak 'Usage: $log->set_pattern( $pattern, $name, $code )';
+
+    my $self    = shift;
+    my $pattern = shift;
+
+    # If no $name is set then we use $pattern as name
+    my ($name, $code) = @_ == 2 ? @_ : ($pattern, @_);
 
     if ($pattern !~ /^%[a-ln-z]\z/i) {
         Carp::croak "invalid pattern '$pattern'";
     }
 
-    if (ref($name) || !length($name)) {
+    if (!defined $name || $name !~ /^[%\w\-\.]+\z/) {
         Carp::croak "invalid/missing name for pattern '$pattern'";
     }
 
-    if (ref($proto) eq 'CODE') {
-        $self->{pattern}->{$pattern}->{code} = $proto;
-    } elsif (length($proto)) {
-        $self->{pattern}->{$pattern}->{code} = $proto;
+    # Structure:
+    #   $self->{pattern}->{'%X'}->{name} = 'name-of-x';
+    #   $self->{pattern}->{'%X'}->{code} = 'value-of-x';
+    $self->{pattern}->{$pattern}->{name} = $name;
+    $self->{pattern}->{$pattern}->{code} = $code;
+}
+
+sub set_level {
+    @_ == 3 or Carp::croak 'Usage: $log->set_level( $alias => { minlevel => $min, maxlevel => $max } )';
+    my ($self, $name, $new) = @_;
+    my $alias = $self->{alias};
+
+    if (!exists $alias->{$name}) {
+        Carp::croak "alias '$name' does not exists";
     }
 
-    $self->{pattern}->{$pattern}->{name} = $name;
+    if (ref($new) ne 'HASH') {
+        Carp::croak "the second parameter to set_level() must be a hash reference";
+    }
+
+    if (!defined $new->{minlevel} && !defined $new->{maxlevel}) {
+        Carp::croak "no new level given to set_level()";
+    }
+
+    foreach my $level (qw/minlevel maxlevel/) {
+        next unless defined $new->{$level};
+
+        if ($new->{$level} =~ LEVEL_RX) {
+            $alias->{$name}->{$level} = $new->{$level};
+            next if $new->{$level} =~ /^\d\z/;
+            $new->{$level} = uc($new->{$level});
+            $new->{$level} = $LEVEL_BY_STRING{ $new->{$level} };
+            $alias->{$name}->{$level} = $new->{$level};
+        } else {
+            Carp::croak "invalid level set to set_level()";
+        }
+    }
+
+    $alias->{$name}->{levels} = { };
+    my $levels = $self->{levels} = { };
+
+    foreach my $level_num ($alias->{$name}->{minlevel} .. $alias->{$name}->{maxlevel}) {
+        my $level = $LEVEL_BY_NUM[ $level_num ];
+        $alias->{$name}->{levels}->{$level} = 1;
+
+        if ($level_num < 4) {
+            $alias->{$name}->{levels}->{FATAL} = 1;
+        }
+    }
+
+    foreach my $output (@{ $self->{outputs} }) {
+        foreach my $level (keys %{$output->{levels}}) {
+            if ($levels->{$level}) {
+                my @old_order = @{$levels->{$level}};
+                $levels->{$level} = [
+                    map  { $_->[0] }
+                    sort { $a->[1] <=> $b->[1] }
+                    map  { [ $_, $_->{priority} ] } @old_order
+                ];
+            } else {
+                push @{$levels->{$level}}, $output;
+            }
+        }
+    }
+
+    return 1;
 }
 
 sub output {
+    @_ == 2 or Carp::croak 'Usage: $log->output( $alias )';
     my ($self, $name) = @_;
-    return unless length($name);
     my $alias = $self->{alias};
     return exists $alias->{$name} ? $alias->{$name}->{output} : undef;
 }
 
+sub flush {
+    my ($self, @alias) = @_;
+    my $errors = ();
+
+    if (@alias) {
+        foreach my $name (@alias) {
+            my $output = $self->output($name);
+            next unless $output;
+            if ( !$output->flush ) {
+                if ( defined $errors ) {
+                    $errors .= '; ' . $output->errstr;
+                } else {
+                    $errors = $output->errstr;
+                }
+            }
+        }
+    } else {
+        foreach my $output (@{$self->{outputs}}) {
+            if ( !$output->flush ) {
+                if ( defined $errors ) {
+                    $errors .= '; ' . $output->errstr;
+                } else {
+                    $errors = $output->errstr;
+                }
+            }
+        }
+    }
+
+    return defined $errors ? $self->_raise_error($errors) : 1;
+}
+
 sub errstr { $ERRSTR }
 
 #
@@ -1001,12 +1388,15 @@ sub _shift_options {
         debug_trace
         die_on_errors
         filter
+        filter_message
+        filter_caller
+        except_caller
         maxlevel
         message_layout
         message_pattern
+        prepare_message
         minlevel
         newline
-        prepare_message
         priority
         timeformat
     );
@@ -1024,7 +1414,7 @@ sub _new_output {
     my $type    = shift;
     my $args    = @_ > 1 ? {@_} : shift;
     my $package = ref($type);
-    my ($output, $options);
+    my ($output, $handler_opts);
 
     # There are two ways to add an output:
     #
@@ -1038,12 +1428,12 @@ sub _new_output {
     #   $log->add(file => \%all_options);
 
     if ( length($package) ) {
-        $output  = $type;
-        $options = $args;
+        $output = $type;
+        $handler_opts = $args;
     } else {
         # Shift the handler options from $args. The rest in %$args
         # will be passed to the output.
-        $options = $self->_shift_options($args);
+        $handler_opts = $self->_shift_options($args);
 
         # Try to determine which output is wanted...
         if (exists $AVAILABLE_OUTPUTS{$type}) {
@@ -1058,16 +1448,21 @@ sub _new_output {
         $output = $package->new($args) or Carp::croak $package->errstr;
     }
 
-    $options = $self->_validate_options($options);
-    return Log::Handler::Output->new($options, $output);
+    $handler_opts = $self->_validate_options($handler_opts);
+    return Log::Handler::Output->new($handler_opts, $output);
 }
 
 sub _validate_options {
-    my $self    = shift;
+    my ($self, @args) = @_;
+    my %wanted  = ();
     my $pattern = $self->{pattern};
-    my (%wanted, $is_fatal);
 
-    my %options = Params::Validate::validate(@_, {
+    # Option 'filter' is deprecated.
+    if (exists $args[0]{filter}) {
+        $args[0]{filter_message} = delete $args[0]{filter};
+    }
+
+    my %options = Params::Validate::validate(@args, {
         timeformat => {
             type => Params::Validate::SCALAR,
             default => '%b %d %H:%M:%S',
@@ -1133,38 +1528,45 @@ sub _validate_options {
             type => Params::Validate::SCALAR,
             optional => 1,
         },
-        filter => {
+        filter_message => {
             type => Params::Validate::SCALAR    # 'foo'
                   | Params::Validate::SCALARREF # qr/foo/
                   | Params::Validate::CODEREF   # sub { shift->{message} =~ /foo/ }
                   | Params::Validate::HASHREF,  # matchN, condition
             optional => 1,
         },
-        caller_level => {
-            type => Params::Validate::SCALAR,
-            regex => NUMB_RX,
-            default => 2,
+        filter_caller => {
+            type => Params::Validate::SCALAR | Params::Validate::SCALARREF,
+            optional => 1,
+        },
+        except_caller => {
+            type => Params::Validate::SCALAR | Params::Validate::SCALARREF,
+            optional => 1,
         },
     });
 
-    if ($options{filter}) {
-        $options{filter} = $self->_validate_filter($options{filter});
+    if ($options{filter_message}) {
+        $options{filter_message} = $self->_validate_filter($options{filter_message});
     }
 
+    # set a default priority if not set
     if (!defined $options{priority}) {
         $options{priority} = $self->{priority}++;
     }
 
+    # replace the level strings with numbers
     foreach my $opt (qw/minlevel maxlevel/) {
         next if $options{$opt} =~ /^\d\z/;
         my $level = uc($options{$opt});
         $options{$opt} = $LEVEL_BY_STRING{$level};
     }
 
+    # iterate from minlevel to maxlevel and create
+    # a hash tree with all active levels
     foreach my $level_num ($options{minlevel} .. $options{maxlevel}) {
         my $level = $LEVEL_BY_NUM[ $level_num ];
         $options{levels}{$level} = 1;
-        next if $is_fatal || $level_num > 3;
+        next if $level_num > 3;
         $options{levels}{FATAL} = 1;
     }
 
@@ -1178,6 +1580,15 @@ sub _validate_options {
             }
             $wanted{$p} = undef;
         }
+
+        # If message_pattern is set to '%T %L %m' then the code
+        # should looks like:
+        #
+        #   sub {
+        #       my ($w, $m) = @_; # %wanted pattern, %message
+        #       $m->{$_} = $w->{$_} for qw/time level message/;
+        #   }
+
         my $func = 'sub { my ($w, $m) = @_; $m->{$_} = $w->{$_} for qw/';
         $func .= join(' ', map { $pattern->{$_}->{name} } keys %wanted);
         $func .= '/ }';
@@ -1191,9 +1602,16 @@ sub _validate_options {
 
         # If the message layout is set to '%T [%L] %m' then the code
         # should looks like:
+        #
         #   sub {
-        #       my $m = shift;
-        #       $m->{time} . ' ]' . $m->{level} . '] ' . $m->{message}
+        #       my ($w, $m) = @_; # %wanted pattern, %message
+        #       $m->{'message'} =
+        #           $w->{'time'}
+        #           . ' ['
+        #           . $w->{'level'}
+        #           . '] '
+        #           . $w->{'message'}
+        #       );
         #   }
 
         foreach my $p ( split /(?:(%[a-zA-Z])|(%)%)/, $options{message_layout} ) {
@@ -1201,7 +1619,7 @@ sub _validate_options {
             if ( exists $pattern->{$p} ) {
                 $wanted{$p} = undef;
                 my $name = $pattern->{$p}->{name};
-                push @chunks, "\$w->{$name}";
+                push @chunks, "\$w->{'$name'}";
             } else {
                 # quote backslash and apostrophe
                 $p =~ s/\\/\\\\/g;
@@ -1211,9 +1629,9 @@ sub _validate_options {
         }
 
         if (@chunks) {
-            $func  = 'sub { my ($w, $m) = @_; $m->{message} = join(\'\', ';
-            $func .= join(', ', @chunks);
-            $func .= ') }';
+            $func  = 'sub { my ($w, $m) = @_; $m->{message} = ';
+            $func .= join('.', @chunks);
+            $func .= ' }';
         }
 
         $options{message_layout_func} = $func;
@@ -1258,15 +1676,36 @@ sub _validate_filter {
         # Each matchN will be checked on the message and the BOOL results
         # will be stored to $filter->{result}->{matchN}. Then the results
         # will be passed to &code. &code returns 0 or 1.
-        #   $filter->{result}->{match1} = $message =~ $filter->{match1}
-        #   $filter->{result}->{match2} = $message =~ $filter->{match2}
-        #   $code = sub { my $m = shift; ($m->{match1} || $m->{match2}) }
+        #
+        # As example if the filter is set to
+        #
+        #   filter => {
+        #       match1    => qr/foo/,
+        #       match2    => qr/bar/,
+        #       condition => '(match1 && match2)',
+        #   }
+        #
+        # Then the bool results will be saved:
+        #
+        #   $filter->{result}->{match1} = $message =~ $filter->{match1};
+        #   $filter->{result}->{match2} = $message =~ $filter->{match2};
+        #
+        # The code for the filter should looks like:
+        #
+        #   $filter->{code} =
+        #       sub {
+        #           my $m = shift;
+        #           ($m->{match1} && $m->{match2})
+        #       }
+        #
         #   &$code($filter->{result});
 
         if (!defined $filter{condition} || $filter{condition} !~ /\w/) {
             Carp::croak "missing condition for paramater 'filter'";
         }
 
+        # Remove all valid characters from the condition
+        # and check if invalid characters left.
         my $cond = $filter{condition};
         $cond =~ s/match\d+//g;
         $cond =~ s/[()&|!<>=\s\d]+//;
@@ -1298,8 +1737,7 @@ sub _validate_filter {
 }
 
 sub _raise_error {
-    my $self = shift;
-    $ERRSTR = shift;
+    $ERRSTR = $_[1];
     return undef;
 }
 
